-- top_level.vhd : prenos dat z desky do SPI
-- Copyright (C) 2011 Brno University of Technology,
--                    Faculty of Information Technology
-- Author(s): Lucie Matusova   xmatus21@stud.fit.vutbr.cz
--

library IEEE;
use ieee.std_logic_1164.ALL;
use ieee.std_logic_ARITH.ALL;
use ieee.std_logic_UNSIGNED.ALL;

architecture main of tlv_gp_ifc is

   signal addr_in  : std_logic_vector (3 downto 0);
   signal data_out : std_logic_vector (7 downto 0);
   signal data_in  : std_logic_vector (7 downto 0);

   signal R        : std_logic;

   signal write_en : std_logic;
   signal read_en  : std_logic;

   -- displej
   signal dis_addr     : std_logic_vector(0 downto 0);
   signal dis_data_out : std_logic_vector(15 downto 0);
   signal dis_write_en : std_logic;

   -- nase   
   signal x_speed  : std_logic_vector(7 downto 0);
   signal y_speed  : std_logic_vector(7 downto 0);
   signal mux_out  : std_logic_vector(7 downto 0);
   
 -----------------------------------------------------------------------------
   -- POUZITE KOMPONENTY

   component SPI_adc
      generic (
         ADDR_WIDTH : integer;   -- parametr určující CELKOVOU šířku adresy
         DATA_WIDTH : integer;   -- pocet bitu dat, ktere se maji prijmout
         ADDR_OUT_WIDTH : integer;
         BASE_ADDR  : integer
      );
      port (
         CLK      : in  std_logic;

         CS       : in  std_logic;
         DO       : in  std_logic;
         DO_VLD   : in  std_logic;
         DI       : out std_logic;
         DI_REQ   : in  std_logic;

         ADDR     : out  std_logic_vector (ADDR_OUT_WIDTH-1 downto 0);
         DATA_OUT : out  std_logic_vector (DATA_WIDTH-1 downto 0);
         DATA_IN  : in   std_logic_vector (DATA_WIDTH-1 downto 0);

         WRITE_EN : out  std_logic;
         READ_EN  : out  std_logic
      );
   end component;

   component lcd_raw_controller
      port (
         RST      : in std_logic;
         CLK      : in std_logic;

         DATA_IN  : in std_logic_vector (15 downto 0);
         WRITE_EN : in std_logic;

         DISPLAY_RS   : out   std_logic;
         DISPLAY_DATA : inout std_logic_vector(7 downto 0);
         DISPLAY_RW   : out   std_logic;
         DISPLAY_EN   : out   std_logic
      );
   end component;


--     nase      -------------------------------------------------------------

  component registr_en
      generic ( 
         n : natural 
      );  
      port (
         CLK : in std_logic;
         RST : in std_logic;
         EN  : in std_logic;
         I   : in std_logic_vector (n-1 downto 0);
         O   : out std_logic_vector(n-1 downto 0) 
      );
  end component;

  component decoder
      generic (
         ADDR_WIDTH : natural;
         EN_WIDTH   : natural
      );
      port (
         INPUT : in std_logic_vector(ADDR_WIDTH-1 downto 0);
         EN    : out std_logic_vector(EN_WIDTH-1 downto 0)  
      );
  end component;

  component mux
      generic ( 
         DATA_WIDTH : natural;
	       ADDR_WIDTH : natural			
      );  
      port (
         I1   : in std_logic_vector (DATA_WIDTH-1 downto 0);
         I2   : in std_logic_vector (DATA_WIDTH-1 downto 0);
			   ADDR : in std_logic_vector (ADDR_WIDTH-1 downto 0);
			   
         Y    : out std_logic_vector(DATA_WIDTH-1 downto 0)  
      );
  end component;

-- ------------------------------------------------------------------------------ 

begin

   LEDF  <= '0';

   SPI_adc_robot: SPI_adc
      generic map(
         ADDR_WIDTH => 8,       -- sirka adresy
         DATA_WIDTH => 8,       -- sirka dat
         ADDR_OUT_WIDTH => 4,   -- sirka adresy na vystupu min. 1 bit - zarizeni
         BASE_ADDR  => 16#80# -- adresovy prostor od 0x80
		-- 0x80 (1000 0000), 0x80 - 0x8F 
      )
      port map(
         CLK   	  =>  CLK,
   
         CS       => SPI_CS,
         DO       => SPI_DO,
         DO_VLD   => SPI_DO_VLD,
         DI       => SPI_DI,
         DI_REQ   => SPI_DI_REQ,
   
         ADDR     => addr_in,
         DATA_OUT => data_out,
         DATA_IN  => data_in,
         WRITE_EN => write_en, 
         READ_EN  => read_en 
      );

   -- SPI dekoder pro displej
   spidecd: SPI_adc
         generic map (
            ADDR_WIDTH => 8,     -- sirka adresy 8 bitu
            DATA_WIDTH => 16,    -- sirka dat 16 bitu
            ADDR_OUT_WIDTH => 1, -- sirka adresy na vystupu 1 bit
            BASE_ADDR  => 16#00# -- adresovy prostor od 0x00-0x01
         )
         port map (
            CLK      => CLK,
            CS       => SPI_CS,

            DO       => SPI_DO,
            DO_VLD   => SPI_DO_VLD,
            DI       => SPI_DI,
            DI_REQ   => SPI_DI_REQ,

            ADDR     => dis_addr,
            DATA_OUT => dis_data_out,
            DATA_IN  => "0000000000000000",
            WRITE_EN => dis_write_en,
            READ_EN  => open
         );

   -- radic LCD displeje
   lcdctrl: lcd_raw_controller
         port map (
            RST    =>  RESET,
            CLK    =>  CLK,

            -- ridici signaly
            DATA_IN  => dis_data_out,
            WRITE_EN => dis_write_en,

            --- signaly displeje
            DISPLAY_RS   => LRS,
            DISPLAY_DATA => LD,
            DISPLAY_RW   => LRW,
            DISPLAY_EN   => LE
         );



  -- registr pro hodnotu
  process (CLK, RESET)
  begin
    if (CLK='1') and (CLK'event) then
      if (RESET='1') then
	      x_speed <= "00000000";
      else
        x_speed(0) <= X(6);
        x_speed(1) <= X(8);
        x_speed(2) <= X(10);
        x_speed(3) <= X(12);
        x_speed(4) <= X(14);
        x_speed(5) <= X(16);
        x_speed(6) <= X(18);
        x_speed(7) <= X(20);
      end if;
    end if;  
  end process;

  -- registr pro adresu
  process (CLK, RESET)
  begin
    if (CLK='1') and (CLK'event) then
      if (RESET='1') then
	      y_speed <= "00000000";
      else
        y_speed(0) <= X(7);
        y_speed(1) <= X(9);
        y_speed(2) <= X(11);
        y_speed(3) <= X(13);
        y_speed(4) <= X(15);
        y_speed(5) <= X(17);
        y_speed(6) <= X(19);
        y_speed(7) <= X(21);
      end if;
    end if;  
  end process;
  
  r_mux: mux generic map (DATA_WIDTH => 8, ADDR_WIDTH => 4)
    port map (I1 => x_speed, I2 => y_speed, ADDR => addr_in, Y => mux_out);

  data_in <= mux_out;

end main;
