/*******************************************************************************
 main.c

 Data prijata z PC jsou zaslana do pinheaderu JP10 a vypsana na LED displej FITkitu.

 Copyright (C) 2013 Brno University of Technology,
 Faculty of Information Technology
 Author(s): Lucie Matusova <xmatus21 AT stud.fit.vutbr.cz>

 *******************************************************************************/

#include <fitkitlib.h> 
#include <keyboard/keyboard.h> 
#include <lcd/display.h>

#define BASE_ADDR 0x81  // adresa, na kt. se poslou data
char char_cnt;          // pocet vypsanych znaku na displeji FITkitu
unsigned int connected; // spojeno s pc?

/*******************************************************************************
 * Vypis uzivatelske napovedy (funkce se vola pri vykonavani prikazu "help")
 *******************************************************************************/
void print_user_help(void) {
    term_send_str_crlf("PC->FITkit->FPGA");
    term_send_str_crlf("FITkit must be properly connected with PC to transmit the data (command OUT).");
}

void confirm_connection() {

}

/*******************************************************************************
 * Dekodovani a vykonani uzivatelskych prikazu
 *******************************************************************************/
unsigned char decode_user_cmd(char *cmd_ucase, char *cmd) {
    unsigned char data;       // 8 bits value from each senzor
    unsigned char send_addr;  // address where we send the data 
    unsigned char sel = 0;        // select - indicates which senzor sends the data
    unsigned char first, second; // auxiliary variable for extracting the senzor value
    
    // set connection with pc    
    if (cmd_ucase[0] == 'O' && cmd_ucase[1] == 'U' && cmd_ucase[2] == 'T') {
        // send confirmation
        term_send_str_crlf("YES");
        // display the confirmation
        if (connected == 0) {
            LCD_append_string("OK");
        }
        
        delay_ms(2000);

        // set flag
        connected = 1;
        return USER_COMMAND;
    }
    
    if (connected == 1) {
       // check "DX" at the beginning!       
       if ((cmd_ucase[0] == 'D') && ((cmd_ucase[1] == 'A') || (cmd_ucase[1] == 'B') ||(cmd_ucase[1] == 'C') ||(cmd_ucase[1] == 'L') ||(cmd_ucase[1] == 'R') ||(cmd_ucase[1] == 'U') ||(cmd_ucase[1] == 'D'))) {
        
          // e.g.: cmd_ucase = "DA0B"
          // extract value, check type of data later
          if (cmd_ucase[2] >= '0' && cmd_ucase[2] <= '9') {
            first = cmd_ucase[2] - '0';
          } else if (cmd_ucase[2] >= 'A' && cmd_ucase[2] <= 'F') {
            first = cmd_ucase[2] - 55; // 10=A, 11=B, ..
          }
          
          if (cmd_ucase[3] >= '0' && cmd_ucase[3] <= '9') {
            second = cmd_ucase[3] - '0';
          } else if (cmd_ucase[3] >= 'A' && cmd_ucase[3] <= 'F') {
            second = cmd_ucase[3] - 55; // 10=A, 11=B, ..
          }      
        
          data = (first << 4) + second;


          // sending data to address 0x81
          // fpga will know what senzor it belongs to by the "select" value
          //  which will be sent to 0x82
          // -> 8 bits, 8 bits select
          /* 
                       _____            __________
          DA,DB,DC,   | ___ |  data    |          |
          DD,DU,DL,DR |l___l|---/8---->|  _       |
          ----------->| ::::|   sel    | |_| fpga |
                      | ::::|---/8---->|          |
                      |_____|          |__________|
          */
          send_addr = BASE_ADDR;

          // senzoric data are displayed
          //  counting chars (maximum 16 chars) and possible display erasing
          if (char_cnt >= 16) {
              LCD_clear();
              char_cnt = 0;
          }

          // determine the type of the senzoric data according to two first chars
          switch (cmd_ucase[0]) {
          case 'D':
              char_cnt += 4;
              // LCD_append_string(cmd_ucase);
              switch (cmd_ucase[1]) {
              case 'A':
                  sel += 0;     
                  break;
              case 'B':
                  sel += 1;
                  break;
              case 'C':
                  sel += 2;
                  break;
              case 'L':
                  sel += 3;
                  break;
              case 'R':
                  sel += 4;
                  break;
              case 'U':         
                  sel += 5;                 
                  break;
              case 'D':
                  sel += 6;               
                  break;
              default:
                  break;
              }
              break;

          default:
              term_send_str_crlf("Wrong parameter!");
              return CMD_UNKNOWN;
          }

          FPGA_SPI_RW_A8_DN(SPI_FPGA_ENABLE_WRITE, send_addr,  &data, 1);
          FPGA_SPI_RW_A8_DN(SPI_FPGA_ENABLE_WRITE, send_addr+1, &sel, 1);

          return USER_COMMAND;
       }
    }
    return CMD_UNKNOWN;
}

/*******************************************************************************
 * Inicialization of peripherals/components after FPGA has been programmed
 *******************************************************************************/
void fpga_initialized() {
    LCD_init();
    LCD_clear();
    LCD_append_string("P-F:");
}

/*******************************************************************************
 * Main
 *******************************************************************************/
int main(void) {
    unsigned int cnt = 0;
    connected = 0;

    char_cnt = 0;

    initialize_hardware();
    keyboard_init();

    set_led_d6(1);
    set_led_d5(1);
    while (1) {
        delay_ms(10);
        cnt++;
        if (cnt > 50) {
            cnt = 0;
            flip_led_d6();
        }

        terminal_idle(); // terminal operation
    }
}
